#include <stdlib.h>
#include <unistd.h>
#include <stdio.h>
#include <errno.h>
#include <string.h>

void zero_plus(char *);

/* structure qui contient la configuration de l'écran (troisième entrée de ligne de l'écran
la fin de l'instruction xrandr --output
et lien vers la prochaîne configuration possible de le même sortie écran */
struct st_cfg
{
 char conf[20];
 char inst[30];
 struct st_cfg * suiv;
};
typedef struct st_cfg c_cfg;

/* structure qui contient le nom de la sortie VGA-TV...
un lien vers la structure qui contient la configuration de l'écran
un lien vers une autre sortie à tester */
struct st_sortie
{
 char nom[20];
 struct st_sortie * suiv;
 c_cfg * cfg;
};
typedef struct st_sortie sortie;

int main(int argc, char **argv)
{
 FILE * ecran;
 char ligne[100];
 char mot1[20],mot2[20],mot3[20];
 sortie * p_e=NULL, * p;
 c_cfg * c;
 char a;

 /* On va sur la racine de l'utilisateur */
 chdir(getenv("HOME"));
 /* puis sur le répertoire ~/.ecran */
 if (chdir(".ecran"))
 {
  /* si ce répertoire n'existe pas */
  if (errno==ENOENT)
  {
   /* on le crée */
   mkdir(".ecran",775);
   /* si cette création est impossible */
   if (chdir(".ecran"))
   {
    /* on signale le problème */
    printf("Erreur création du répertoire ~/.ecran non effective\n");
    /* et on sort */
    return 1;
   }
   else
   {
    printf("Création du répertoire ~/.ecran.\n Vous devez y mettre le fichier ecran.cfg de configuration.\n");
    return 0;
   }
  }
  /* on ne peut pas aller sur le répertoire ~/.ecran avec une erreur qui ne signale pas son absence */
  else
  {
   /* on le signale */
   printf("Erreur pour aller sur le répertoire ~/.ecran\n");
   /* on sort du progamme */
   return 1;
  }
 }
 /* Juste pour connaître les écrans activés et les mettre en mode auto */
 system("xrandr > ecran.txt");
 /* ouverture de la sortie de xrandr pour analyse */
 ecran=fopen("ecran.txt","r");
 /* parcours des lignes unes à une */
 while (fgets(ligne,100,ecran)!=NULL)
 {
   /* lecture des trois premiers mots de la ligne */
   sscanf(ligne,"%s %s %s",mot1,mot2,mot3);
   /* On vérifie si c'est une sortie connectée */
   if (strcmp(mot2,"connected")==0)
   {
    sprintf(ligne,"xrandr --output %s --auto",mot1);
    printf("%s\n",ligne);
    system(ligne);
   }
 }
 fclose(ecran);
 /* on met dans le fichier ~/.ecran/ecran.txt la sortie de xrandr */
 system("xrandr > ecran.txt");

 /* on ouvre le fichier de configuration qui indique les instructions à faire en fonction de la configuration */
 if ((ecran=fopen("ecran.cfg","r"))==NULL)
 {
  printf("Ouverture de ~/.ecran/ecran.cfg impossible\nq");
  return 1;
 }
 /* parcours des lignes les unes après les autres */
 while (fgets(ligne,100,ecran)!=NULL)
 {
  sscanf(ligne,"%c",&a);
  /* on évite les lignes de commentaires */
  if (a!='#')
  {
  /* lecture des trois premiers mots de la ligne */
  sscanf(ligne,"%s %s",mot1,mot2);
  zero_plus(mot2);
  p=p_e;
  /* recherche si on a déjà cette sortie de configuré */
  while (p!=NULL && strcmp(p->nom,mot1)!=0)
   p=p->suiv;
  /* si la sortie n'est pas configurée au moins une fois, on chaîne cette sortie au début */
  if (p==NULL)
  {
   p=p_e;
   p_e=(sortie *)malloc(sizeof(sortie));
   p_e->suiv=p;
   strcpy(p_e->nom,mot1);
   p_e->cfg=(c_cfg *)malloc(sizeof(c_cfg));
   p_e->cfg->suiv=NULL;
   strcpy(p_e->cfg->conf,mot2);
   strcpy(p_e->cfg->inst,ligne+strlen(mot1)+strlen(mot2)+2);
  }
  /* si la sortie existe, on chaine cette configuration au début */
  else
  {
   c=p->cfg;
   p->cfg=(c_cfg *)malloc(sizeof(c_cfg));
   p->cfg->suiv=c;
   strcpy(p->cfg->conf,mot2);
   strcpy(p->cfg->inst,ligne+strlen(mot1)+strlen(mot2)+2);
  }
  }
 }
 fclose(ecran);

 /* ouverture de la sortie de xrandr pour analyse */
 ecran=fopen("ecran.txt","r");
 /* parcours des lignes unes à une */
 while (fgets(ligne,100,ecran)!=NULL)
 {
   /* lecture des trois premiers mots de la ligne */
   sscanf(ligne,"%s %s %s",mot1,mot2,mot3);
   /* On vérifie si c'est une sortie connectée */
   if (strcmp(mot2,"connected")==0)
   {
    /* si oui, on cherche cette sortie dans la chaine des p_e */
    p=p_e;
    while(p!=NULL && strcmp(p->nom,mot1)!=0)
     p=p->suiv;
    /* si on e trouvé cette sortie */
    if (p!=NULL)
    {
     zero_plus(mot3);
     c=p->cfg;
     /* on cherche si on a une instruction pour cette configuration */
     while(c!=NULL && strcmp(c->conf,mot3)!=0)
      c=c->suiv;
     /* si non, on le signale */
     if (c==NULL)
     {
      printf("Configuration %s non prise en compte\n",ligne);
     }
     /* si oui, on fait l'instruction */
     else
     {
      printf("instruction : xrandr --output %s\n",c->inst);
      sprintf(ligne,"xrandr --output %s",c->inst);
      system(ligne);
     }
    }
    /* Si aucune configuration ne dépend de l'état de cette sortie */
    else
     printf("Sortie %s non prise en compte\n",mot1);
   }
 }
 fclose(ecran);
}

void zero_plus(char * mot)
{
 while (*mot && *mot!='+')
  mot++;
 *mot=0;
}
